# -*- coding: utf-8 -*-
#Make changes for python 2 and 3 compatibility
try:
    import builtins     # if this success it's python 3.x
    builtins.xrange = range
    builtins.basestring = (str,bytes)
except ImportError:
    pass                # python 2.x

#Find suffix
import sysconfig
suffix = sysconfig.get_config_var('EXT_SUFFIX')
if suffix is None:
    suffix = ".so"

#Import shared C library
import os
pymodulespath = os.path.dirname(__file__)
from ctypes import *
clibreboundx = cdll.LoadLibrary(pymodulespath + '/../libreboundx' + suffix)

# Get string for build in libreboundx.so
def build_date():
    return c_char_p.in_dll(clibreboundx, "rebx_build_str").value.decode("ascii")
def build_version():
    return c_char_p.in_dll(clibreboundx, "rebx_version_str").value.decode("ascii")
#Check for version
try:
    moduleversion = pkg_resources.require("reboundx")[0].version
    libreboundxversion = build_version()
    if moduleversion != libreboundxversion:
        print("WARNING: python module and libreboundx have different version numbers: '%s' vs '%s'.\n".format(moduleversion, libreboundxversion))
except:
    pass    # this check fails in python 3. Problem with setuptools

# When reboundx is imported, first monkey patch rebound.Particle so that we can add new parameters to the particles:

import rebound
def monkeyset(self, name, value):
    if (name not in rebound.Particle.__dict__) and (not hasattr(super(rebound.Particle, self), name)):
        # create new param in c
        clibreboundx.rebx_set_param_double(byref(self), c_char_p(name.encode('utf-8')), c_double(value))
    else:
        rebound.Particle.default_set(self, name, value)
        
def monkeyget(self, name):
    if (name not in rebound.Particle.__dict__) and (not hasattr(super(rebound.Particle, self), name)):
        # check param in c
        clibreboundx.rebx_get_param_double.restype = c_double
        return clibreboundx.rebx_get_param_double(byref(self), c_char_p(name.encode('utf-8')))
    else:
        return super(rebound.Particle, self).__getattr__(name)

rebound.Particle.default_set = rebound.Particle.__setattr__
rebound.Particle.__setattr__ = monkeyset
rebound.Particle.__getattr__ = monkeyget

from .extras import Extras
from .tools import coordinates, install_test
__all__ = ["Extras", "coordinates", "install_test"]



# -*- coding: utf-8 -*-

"""
Initial conditions for standard tests

"""

import math

def add_earths(sim, ei):
    """
    Add a simple test system with 2 ~ Earths, with eccentricities and inclinations (same) specified
    """
    sim.add(m=1.)
    massratio = 3.e-6
    sim.add(m=massratio, a=1., e=ei, inc=ei)
    sim.add(m=massratio, a=3., e=ei, inc=ei)



from . import clibreboundx
from ctypes import Structure, c_double, POINTER, c_int, c_uint, c_long, c_ulong, c_void_p, c_char_p, CFUNCTYPE, byref, c_uint32
import rebound

C_DEFAULT = 10064.915 # speed of light in default units (G = 1) of AU / (yr/2pi)

class Extras(Structure):
    """
    Main object used for all REBOUNDx operations, tied to a particular REBOUND simulation.
    This is an abstraction of the C struct rebx_extras, with all the C convenience functions
    and functions for adding effects implemented as methods of the class.  
    The fastest way to understand it is to follow the examples at :ref:`ipython_examples`.  
    """

    def __init__(self, sim):
        clibreboundx.rebx_initialize(byref(sim), byref(self)) # Use memory address ctypes allocated for rebx Structure in C
        sim._extras_ref = self # add a reference to this instance in sim to make sure it's not garbage collected

    def __del__(self):
        if self._b_needsfree_ == 1:
            clibreboundx.rebx_free_pointers(byref(self))
    
    #######################################
    # Functions for adding REBOUNDx effects
    #######################################

    def add_modify_orbits_direct(self):
        """
        :rtype: rebx_params_modify_orbits_direct
        """
        clibreboundx.rebx_add_modify_orbits_direct.restype = POINTER(rebx_params_modify_orbits_direct)
        return clibreboundx.rebx_add_modify_orbits_direct(byref(self)).contents

    def add_modify_orbits_forces(self):
        """
        :rtype: rebx_params_modify_orbits_forces
        """
        clibreboundx.rebx_add_modify_orbits_forces.restype = POINTER(rebx_params_modify_orbits_forces)
        return clibreboundx.rebx_add_modify_orbits_forces(byref(self)).contents

    def add_gr(self, source_index=0, c=C_DEFAULT): 
        """
        You must pass c (the speed of light) in whatever units you choose if you don't use default units of AU, (yr/2pi) and Msun.

        :param source: Source of GR effect. 
        :param c: Speed of light in appropriate units.
        :type source: rebound.Particle
        :type c: double
        :rtype: rebx_params_gr
        """
        clibreboundx.rebx_add_gr.restype = POINTER(rebx_params_gr)
        return clibreboundx.rebx_add_gr(byref(self), c_int(source_index), c_double(c)).contents # Sets source to particles[0] in C code when passed NULL (=None)
    
    def add_gr_full(self, c=C_DEFAULT):
        """
        You must pass c (the speed of light) in whatever units you choose if you don't use default units of AU, (yr/2pi) and Msun.
        
        :param c: Speed of light in appropriate units.
        :type c: double
        :rtype: rebx_params_gr_full
        """
        clibreboundx.rebx_add_gr_full.restype = POINTER(rebx_params_gr_full)
        return clibreboundx.rebx_add_gr_full(byref(self), c_double(c)).contents

    def add_gr_potential(self, source_index=0, c=C_DEFAULT):
        """
        You must pass c (the speed of light) in whatever units you choose if you don't use default units of AU, (yr/2pi) and Msun.
        
        :param source: Source of GR effect.
        :param c: Speed of light in appropriate units.
        :type source: rebound.Particle
        :type c: double
        :rtype: rebx_params_gr_potential
        """
        clibreboundx.rebx_add_gr_potential.restype = POINTER(rebx_params_gr_potential)
        return clibreboundx.rebx_add_gr_potential(byref(self), c_int(source_index), c_double(c)).contents
    
    def add_radiation_forces(self, source_index=0, c=C_DEFAULT):
        """
        You must pass c (the speed of light) in whatever units you choose if you don't use default units of AU, (yr/2pi) and Msun.
        
        :param source: Source of radiation. 
        :param c: Speed of light in appropriate units.
        :type source: rebound.Particle
        :type c: double
        :rtype: rebx_params_radiation_forces
        """
        clibreboundx.rebx_add_radiation_forces.restype = POINTER(rebx_params_radiation_forces)
        return clibreboundx.rebx_add_radiation_forces(byref(self), c_int(source_index), c_double(c)).contents
    
    #######################################
    # Convenience Functions
    #######################################

    def rad_calc_beta(self, params, particle_radius, density, Q_pr, L):

        """
        Calculates a particle's beta parameter (the ratio of the radiation force to the gravitational force).
        All values must be passed in the same units as used for the simulation as a whole (e.g., AU, Msun, yr/2pi).

        :param params: parameters instance returned by add_radiation_forces.
        :param particle_radius: grain's physical radius
        :param density: particle bulk density
        :param Q_pr: radiation pressure coefficient
        :param L: Radiation source's luminosity
        :type params: rebx_params_radiation_forces
        :type particle_radius: float
        :type density: float
        :type Q_pr: float
        :type L: float
        :rtype: float
        """
        clibreboundx.rebx_rad_calc_beta.restype = c_double
        return clibreboundx.rebx_rad_calc_beta(byref(self), byref(params), c_double(particle_radius), c_double(density), c_double(Q_pr), c_double(L))
    def rad_calc_particle_radius(self, params, beta, density, Q_pr, L):
        """
        Calculates a particle's physical radius given its beta parameter.
        All values must be passed in the same units as used for the simulation as a whole (e.g., AU, Msun, yr/2pi).

        :param params: parameters instance returned by add_radiation_forces.
        :param beta: ratio of radiation pressure force to gravitational force from the radiation source.
        :param density: particle bulk density
        :param Q_pr: radiation pressure coefficient
        :param L: Radiation source's luminosity
        :type params: rebx_params_radiation_forces
        :type beta: float
        :type density: float
        :type Q_pr: float
        :type L: float
        :rtype: float
        """
        clibreboundx.rebx_rad_calc_particle_radius.restype = c_double
        return clibreboundx.rebx_rad_calc_particle_radius(byref(self), byref(params), c_double(beta), c_double(density), c_double(Q_pr), c_double(L))
    
#######################################
# Effect parameter class definitions
#######################################

class rebx_params_modify_orbits_direct(Structure):
    _fields_ = [("p", c_double),
                ("coordinates", c_int)]

class rebx_params_modify_orbits_forces(Structure):
    _fields_ = [("coordinates", c_int)]

class rebx_params_gr(Structure):
    _fields_ = [("source_index", c_int),
                ("c", c_double)]

class rebx_params_gr_potential(Structure):
    _fields_ = [("source_index", c_int), # testing
                ("c", c_double)]

class rebx_params_gr_full(Structure):
    _fields_ = [("c", c_double)]

class rebx_params_radiation_forces(Structure):
    _fields_ = [("source_index", c_int),
                ("c", c_double)]

#################################################
# Generic REBOUNDx definitions
#################################################

class rebx_param(Structure): # need to define fields afterward because of circular ref in linked list
    pass    
rebx_param._fields_ = [("paramPtr", c_void_p),
            ("param_type", c_int),
            ("next", POINTER(rebx_param))]

class rebx_effect(Structure):
    pass
rebx_effect._fields_ = [("paramsPtr", c_void_p),
                        ("functionPtr", CFUNCTYPE(None, POINTER(rebound.Simulation), POINTER(rebx_effect))),
                        ("effect_type", c_uint32),
                        ("is_force", c_int),
                        ("next", POINTER(rebx_effect))]

class rebx_param_to_be_freed(Structure):
    pass
rebx_param_to_be_freed._fields_ = [("param", POINTER(rebx_param)),
            ("next", POINTER(rebx_param_to_be_freed))]


# Need to put fields after class definition because of self-referencing
Extras._fields_ = [("sim", POINTER(rebound.Simulation)),
                ("effects", POINTER(rebx_effect)),
                ("params_to_be_freed", POINTER(rebx_param_to_be_freed))]



try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension
from codecs import open
import os
import inspect
import sys 

import sysconfig
suffix = sysconfig.get_config_var('EXT_SUFFIX')
if suffix is None:
    suffix = ".so"

try:
    import rebound
except ImportError:
    print("You must first install REBOUND.  See http://rebound.readthedocs.org/en/latest/python_quickstart.html")
    sys.exit(1)

from distutils.version import LooseVersion
if LooseVersion(rebound.__version__) < LooseVersion("2.12.1"): # Only after 2.12.1 can you link to librebound.so 
    print("REBOUNDx requires REBOUND version > 2.12.1.  Please upgrade.  See 5.3 in http://rebound.readthedocs.org/en/latest/python_quickstart.html")
    sys.exit(1)

rebdir = os.path.dirname(inspect.getfile(rebound))

extra_link_args=[]
if sys.platform == 'darwin':
    from distutils import sysconfig
    vars = sysconfig.get_config_vars()
    vars['LDSHARED'] = vars['LDSHARED'].replace('-bundle', '-shared')
    extra_link_args=['-Wl,-install_name,@rpath/libreboundx'+suffix]
    extra_link_args.append('-Wl,-rpath,'+rebdir+'/../')

libreboundxmodule = Extension('libreboundx',
                    sources = [ 'src/core.c', 'src/gr.c', 'src/gr_full.c', 'src/gr_potential.c', 'src/modify_orbits_direct.c', 'src/modify_orbits_forces.c', 'src/radiation_forces.c', 'src/rebxtools.c'],
                    include_dirs = ['src', rebdir],
                    library_dirs = [rebdir+'/../'],
                    runtime_library_dirs = [rebdir+'/../'],
                    libraries=['rebound'+suffix[:-3]], #take off .so from the suffix
                    define_macros=[ ('LIBREBOUNDX', None) ],
                    extra_compile_args=['-fstrict-aliasing', '-O3','-std=c99','-march=native', '-fPIC', '-Wpointer-arith'],
                    extra_link_args=extra_link_args,
                    )

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='reboundx',
    version='2.6.0',
    description='A library for including additional forces in REBOUND',
    long_description=long_description,
    url='http://github.com/dtamayo/reboundx',
    author='Daniel Tamayo',
    author_email='tamayo.daniel@gmail.com',
    license='GPL',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Scientific/Engineering :: Astronomy',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    keywords='astronomy astrophysics nbody integrator',
    packages=['reboundx'],
    #install_requires=['rebound>=2.12.0'],
    tests_require=["numpy","matplotlib"],
    test_suite="reboundx.test",
    ext_modules = [libreboundxmodule],
    zip_safe=False)
